#!/usr/bin/python3

######
## This script takes a json file of paired issues and translates the attachment and comment ids to the migrated system.
######

import requests, json, sys, getopt
from functools import reduce

### parsing command line arguments
def getOptions(argv):
	def printUsage():
		print('test.py -f <mappingFile> --bu1 <urlToFirstJira> --u1 <userFirstJira> --p1 <passwordFirstJira> --bu2 <urlToSecondJira> --u2 <userSecondJira> --p2 <passwordSecondJira>')
		print('If the lookup needs to be performed on the secondConnector, please specify the "-r" option for reversing the connectors.')

	try:
		opts, args = getopt.getopt(argv,"hrf:",["file=","bu1=","u1=","p1=","bu2=","u2=","p2=", "baseUrl1=","user1=","password1=","baseUrl2=","user2=","password2="])
	except getopt.GetoptError as e:
		print(str(e))
		printUsage()
		sys.exit(2)

	reverseConnectors = False

	for opt, arg in opts:
		if opt == '-h':
			printUsage()
			sys.exit()
		elif opt == '-r':
			reverseConnectors = True
		elif opt in ("-f", "--file"):
			mappingFile = arg
		elif opt in ("--bu1", "--baseUrl1"):
			baseUrl1 = arg
		elif opt in ("--u1", "--user1"):
			user1 = arg
		elif opt in ("--p1", "--password1"):
			password1 = arg
		elif opt in ("--bu2", "--baseUrl2"):
			baseUrl2 = arg
		elif opt in ("--u2", "--user2"):
			user2 = arg
		elif opt in ("--p2", "--password2"):
			password2 = arg
	if 'mappingFile' not in locals():
		print('mapping file is missing')
		printUsage()
		sys.exit(2)
	if 'baseUrl1' not in locals() or 'user1' not in locals() or 'password1' not in locals():
		print('connection information for first Jira is missing')
		printUsage()
		sys.exit(2)
	if 'baseUrl2' not in locals() or 'user2' not in locals() or 'password2' not in locals():
		print('connection information for second Jira is missing')
		printUsage()
		sys.exit(2)

	return mappingFile, reverseConnectors, baseUrl1, user1, password1, baseUrl2, user2, password2


### checking if the data for this connector contains deleted/not visible issues
def processConnector(connectorMappingData, baseUrl, user, password, baseUrl2, user2, password2):
	def getIdKey(attachmentOrComent):
		return attachmentOrComent['id']

	def getFilenameCreatedKey(attachment):
		return f'{attachment["filename"]}-{attachment["created"]}'

	def getBodyCreatedKey(comment):
		return f'{comment["body"]}-{comment["created"]}'

	mappingsPerIssue = {}
	# iterate over firstConnector
	for issueData in connectorMappingData:
		localIssueKey = issueData["localIssueKey"]
		print(f'processing {localIssueKey}')

		firstStatusCode, firstAttachmentsMap, firstCommentsMap = getAttachmentsAndComments(localIssueKey, getIdKey, getIdKey, baseUrl, user, password)
		if firstStatusCode == 200:
			secondStatusCode, secondAttachmentsMap, secondCommentsMap = getAttachmentsAndComments(localIssueKey, getFilenameCreatedKey, getBodyCreatedKey, baseUrl2, user2, password2)

			if secondStatusCode == 200:
				attachmentIdMapping = {}
				# Iterate over attachments in firstAttachmentsMap
				for firstAttachmentId, firstAttachment in firstAttachmentsMap.items():
					filename = getFilenameCreatedKey(firstAttachment)
					# Check if filename exists in secondAttachmentsMap
					if filename in secondAttachmentsMap:
						secondAttachment = secondAttachmentsMap[filename]
						secondAttachmentId = secondAttachment['id']
						attachmentIdMapping[firstAttachmentId] = secondAttachmentId
					else:
						print(f'No mapping found for attachment with filename: {filename}')

				if 'attachments' in issueData:
					for attachment in issueData['attachments']:
						if 'localId' in attachment:
							localId = attachment['localId']
							if localId in attachmentIdMapping:
								print(f'Mapping attachment: {localId} --> {attachmentIdMapping[localId]}')
								attachment['localId'] = attachmentIdMapping[localId]
							else:
								print(f'No attachment found with localId: {localId}')

				commentIdMapping = {}
				# Iterate over comments in firstCommentsMap
				for firstCommentId, firstComment in firstCommentsMap.items():
					body = getBodyCreatedKey(firstComment)
					# Check if body exists in secondCommentsMap
					if body in secondCommentsMap:
						secondComment = secondCommentsMap[body]
						secondCommentId = secondComment['id']
						commentIdMapping[firstCommentId] = secondCommentId
					else:
						print(f'No mapping found for comment with body: {body}')

				if 'comments' in issueData:
					for comment in issueData['comments']:
						if 'localId' in comment:
							localId = comment['localId']
							if localId in commentIdMapping:
								print(f'Mapping comment: {localId} --> {commentIdMapping[localId]}')
								comment['localId'] = commentIdMapping[localId]
							else:
								print(f'No comment found with localId: {localId}')

				mappingsPerIssue[localIssueKey] = {"attachmentIdMapping": attachmentIdMapping, "commentIdMapping": commentIdMapping}
	return mappingsPerIssue


def getAttachmentsAndComments(localIssueKey, attachmentKeyFunction, commentKeyFunction, baseUrl, user, password):
	issueResponse = requests.get(f"{baseUrl}/rest/api/2/issue/{localIssueKey}?fields=attachment,comment", auth=(user, password))
	# print(f'response {issueResponse.text}')
	status_code = issueResponse.status_code

	if status_code == 200:
		issue = issueResponse.json()

		attachmentsMap = {}
		for attachment in issue['fields']['attachment']:
			#print(f'attachment with id {attachment["id"]} and name {attachment["filename"]}')
			attachmentsMap[attachmentKeyFunction(attachment)] = attachment

		commentsMap = {}
		for comment in issue['fields']['comment']['comments']:
			#print(f'comment with id {comment["id"]} and name {comment["body"]}')
			commentsMap[commentKeyFunction(comment)] = comment

		#print(f'attachments map: {attachmentsMap}')
		#print(f'comments map: {commentsMap}')
		return status_code, attachmentsMap, commentsMap
	else:
		print(f'issue {localIssueKey} not found in {baseUrl}')
		return status_code, None, None

def replaceRemoteIds(connectorMappingData, mappingsPerIssue):
	for issueData in connectorMappingData:
		print(f'processing {issueData["localIssueKey"]}')
		remoteIssueKey = issueData['remoteIssueKey']

		if remoteIssueKey in mappingsPerIssue:
			mappingPerIssue = mappingsPerIssue[remoteIssueKey]
			attachmentIdMapping = mappingPerIssue['attachmentIdMapping']
			commentIdMapping = mappingPerIssue['commentIdMapping']

			if 'attachments' in issueData:
				for attachment in issueData['attachments']:
					if 'remoteId' in attachment:
						remoteId = attachment['remoteId']
						if remoteId in attachmentIdMapping:
							print(f'Mapping attachment: {remoteId} --> {attachmentIdMapping[remoteId]}')
							attachment['remoteId'] = attachmentIdMapping[remoteId]
						else:
							print(f'No attachment found with id: {remoteId}')

			if 'comments' in issueData:
				for comment in issueData['comments']:
					if 'remoteId' in comment:
						remoteId = comment['remoteId']
						if remoteId in commentIdMapping:
							print(f'Mapping comment: {remoteId} --> {commentIdMapping[remoteId]}')
							comment['remoteId'] = commentIdMapping[remoteId]
						else:
							print(f'No comment found with id: {remoteId}')

def saveJson(jsonData, filename):
	print(f'saving json to {filename}')

	with open(filename, 'w') as f:
		json.dump(jsonData, f, indent=4)

### start of script
mappingFile, reverseConnectors, baseUrl1, user1, password1, baseUrl2, user2, password2 = getOptions(sys.argv[1:])

#parse mapping file
with open(mappingFile) as f:
	mappingData = json.load(f)
	#print(mappingData)

if reverseConnectors:
	connector1 = "secondConnector"
	connector2 = "firstConnector"
else:
	connector1 = "firstConnector"
	connector2 = "secondConnector"

print(f'### Processing sync infos for {connector1}')
mappingsPerIssue = processConnector(mappingData[connector1], baseUrl1, user1, password1, baseUrl2, user2, password2)

if connector2 in mappingData:
	print(f'### Replacing remote ids for {connector2}')
	replaceRemoteIds(mappingData[connector2], mappingsPerIssue)

saveJson(mappingData, mappingFile + '-converted.json')
